<?php

namespace App\Console\Commands\Flow;

use App\Console\Commands\Star\VideoPostOrderDataSync;
use App\Models\JuxingAdAccount;
use App\Models\JxStarVideoFlowOrderDailyData;
use App\Models\JxStarVideoFlowOrderHourData;
use App\Models\JxStarVideoOrderList;
use App\Support\EmailQueue;
use App\Support\Log;
use App\Support\RedisModel;
use Illuminate\Console\Command;
use kwaiSDK\JuXing;

class FlowStarVideoOrderDailyDataSync extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'Flow:StarVideoOrderDailyDataSync';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = '流量助推星视频订单每日数据获取';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $this->info(date('H:i') . ' 开始执行');
        $this->start();
        $this->info(date('H:i') . ' 结束执行');

        return 0;
    }

    public function start()
    {
        $curTime = time();
        while (true) {
            if ((time() - $curTime) >= 1200) break;
            $len = RedisModel::lLen(JuxingAdAccount::JUXING_FLOW_STAR_VIDEO_DAILY_SYNC_ADVID_LIST);
            if($len > 1000) {
                EmailQueue::rPush('流量助推星视频订单每日数据获取队列堆积', $len, ['xiaohua.hou@kuxuan-inc.com'], '聚星');
            }

            $dataJson = RedisModel::rPop(JuxingAdAccount::JUXING_FLOW_STAR_VIDEO_DAILY_SYNC_ADVID_LIST);
            if (empty($dataJson)) {
                sleep(2);
                continue;
            }

            $data = json_decode($dataJson, true);
            if (!isset($data['advertiserId'], $data['timeRangeArr'])) {
                # 输出错误日志
                Log::error('数据参数不合法', $data, 'Flow:StarVideoOrderDailyDataSync');
                continue;
            }

            $this->getDailyData($data);

            // 控频
            sleep(1);
        }
    }

    public function getDailyData($data)
    {
        $advertiserId = $data['advertiserId'];
        $timeRangeArr = $data['timeRangeArr'];

        $accessToken = JuxingAdAccount::getAccessToken($advertiserId);
        $juxing = new JuXing($accessToken);

        $curDate = date('Y-m-d');
        $lastDate = date('Y-m-d', strtotime("-1 day"));
        $ago7Date = date('Y-m-d', strtotime("-7 day"));
        $curHour = date('Y-m-d H:00:00');
        $lastHour = date('Y-m-d H:00:00', strtotime('-1 hour'));

        # 获取星视频(昨天之后创建的视频需要插入更新)
        $starVideoList = $juxing->investData()->taskList([
            'advertiser_id'     => $advertiserId,
            'create_start_time' => $ago7Date,
            'create_end_time'   => $curDate
        ]);

        if (!isset($starVideoList['code']) || ($starVideoList['code'] != 0)) {
            # 输出错误日志并异常处理
            $msg = $rst['message'] ?? null;
            Log::error('获取星视频列表请求接口错误:'.$msg, [
                'advertiserId' => $advertiserId,
                'timeRange'    => [
                    'st' => $ago7Date,
                    'en' => $curDate
                ],
                'res' => $starVideoList
            ], 'Flow:StarVideoOrderDailyDataSync');
            return false;
        }

        $starVideoList = $starVideoList['data']['details'] ?? [];
        if (!empty($starVideoList)) {
            foreach ($starVideoList as $starVideo) {
                (new VideoPostOrderDataSync())->allotmentData($advertiserId, $starVideo);
            }
        }

        $totalOrderCnt = 0;
        foreach ($timeRangeArr as $timeRange) {
            # 获取流量助推星视频
            $param = [
                'advertiser_id'          => $advertiserId,
                'create_start_time'      => $timeRange['st'],
                'create_end_time'        => $timeRange['en'],
                'supplement_order_type'  => 1, //星视频
            ];

            $flowStarVideoOrderSync = new FlowStarVideoOrderSync();
            $listGenerator = $flowStarVideoOrderSync->generator($juxing, $param);

            foreach ($listGenerator as $list) {
                # 统计订单数
                $totalOrderCnt += count($list);

                $starMap     = $this->getStarMap($advertiserId, $list);
                $lastDataMap = $this->getLastDataMap($advertiserId, $lastDate, $list);
                $lastHourMap = $this->getLastHourDataMap($advertiserId, $lastHour, $list);
                foreach ($list as $dataVal) {
                    # 更新每日订单数据
                    $this->orderDailyDataInDb($advertiserId, $dataVal, $curDate, $starMap, $lastDataMap);
                    # 更新每小时订单数据
                    $this->orderHourDataInDb($advertiserId, $dataVal, $curHour, $starMap, $lastHourMap);
                }
            }
        }

        Log::info('avd:'.$advertiserId.' 获取订单数:'.$totalOrderCnt, [], 'Flow:StarVideoOrderDailyDataSync');
    }

    public function getStarMap($advertiserId, $list)
    {
        $taskIds = array_unique(array_column($list, 'task_id'));
        $orderIds = array_unique(array_column($list, 'order_id'));

        # 查网红信息
        $starList = JxStarVideoOrderList::query()
            ->select('task_id', 'order_id', 'star_name', 'kwai_id', 'user_id')
            ->where('advertiser_id', $advertiserId)
            ->whereIn('task_id', $taskIds)
            ->whereIn('order_id', $orderIds)
            ->where('enable', 1)
            ->get();

        $starMap = [];
        if ($starList->isNotEmpty()) {
            foreach ($starList as $starInfo) {
                if (!isset($starMap[$starInfo->task_id][$starInfo->order_id])) {
                    $starMap[$starInfo->task_id][$starInfo->order_id] = [
                        'star_name' => $starInfo->star_name,
                        'kwai_id'   => $starInfo->kwai_id,
                        'user_id'   => $starInfo->user_id
                    ];
                }
            }
        }

        return $starMap;
    }

    public function getLastDataMap($advertiserId, $lastDate, $list)
    {
        $supplementOrderIsd = array_column($list, 'supplement_order_id');
        return JxStarVideoFlowOrderDailyData::query()
            ->where('advertiser_id', $advertiserId)
            ->where('date', $lastDate)
            ->whereIn('supplement_order_id', $supplementOrderIsd)
            ->where('enable', 1)
            ->get()
            ->keyBy('supplement_order_id');
    }

    public function getLastHourDataMap($advertiserId, $lastHour, $list) {
        $supplementOrderIsd = array_column($list, 'supplement_order_id');
        return JxStarVideoFlowOrderHourData::query()
            ->where('advertiser_id', $advertiserId)
            ->where('date', $lastHour)
            ->whereIn('supplement_order_id', $supplementOrderIsd)
            ->where('enable', 1)
            ->get()
            ->keyBy('supplement_order_id');
    }

    public function orderDailyDataInDb($advertiserId, $dataVal, $curDate, $starMap, $lastDataMap)
    {
        # 检测订单数据
        $orderId = $dataVal['order_id'] ?? null;
        if (empty($orderId)) {
            # 输出错误日志
            Log::error('adv:'.$advertiserId.' orderId 为空', $dataVal, 'Flow:StarVideoOrderDailyDataSync');
            return false;
        }

        # 检测生效助推订单数
        $closingTime = $dataVal['closing_time'];
        if (
            !empty($closingTime)
            &&
            ($closingTime < date('Y-m-d H:i:s', strtotime("-12 hour")))
        ) {
            return true;
        }

        # 获取达人信息
        $taskId = $dataVal['task_id'];
        $starInfo = $starMap[$taskId][$orderId] ?? [
                'kwai_id'   => 'None',
                'user_id'   => -1,
                'star_name' => 'None'
            ];

        # 检测是否存在当天数据
        $supplementOrderId = $dataVal['supplement_order_id'];
        $cData = JxStarVideoFlowOrderDailyData::query()
            ->where('date', $curDate)
            ->where('advertiser_id', $advertiserId)
            ->where('supplement_order_id', $supplementOrderId)
            ->first();

        # 数据异常检测
        $consumeAmount = $dataVal['consume_amount'];
        $play          = $dataVal['play'] ?? 0;
        $action        = $dataVal['action'] ?? 0;
        $conversion    = $dataVal['conversion'] ?? 0;
        if (!empty($cData)) {
            if (
                ($consumeAmount < $cData->consume_amount_tal)
                ||
                ($play < $cData->play_tal)
                ||
                ($action < $cData->action_tal)
                ||
                ($conversion < $cData->conversion_tal)
            )
            {
                # 输出错误日志
                Log::info('adv:'.$advertiserId.' 返回数据存在异常', $dataVal, 'Flow:StarVideoOrderDailyDataSync');
                return false;
            }
        } else {
            $cData = new JxStarVideoFlowOrderDailyData();
            $cData->advertiser_id       = $advertiserId;
            $cData->task_id             = $taskId;
            $cData->supplement_order_id = $supplementOrderId;
            $cData->date                = $curDate;
        }

        # 达人数据更新
        if (($cData->user_id == -1) || is_null($cData->user_id)) {
            $cData->kwai_id   = $starInfo['kwai_id'];
            $cData->user_id   = $starInfo['user_id'];
            $cData->star_name = $starInfo['star_name'];
        }

        # 其他数据
        $played3sPercent   = isset($dataVal['played3s_percent'])
            ? round(floatval($dataVal['played3s_percent'])/100, 4) : 0;
        $view              = $dataVal['view'] ?? 0;
        $comment           = $dataVal['comment'] ?? 0;
        $admire            = $dataVal['admire'] ?? 0;
        $played3sCnt       = round($played3sPercent * $play);
        $submissionsNumber = $dataVal['submissions_number'] ?? 0;

        # 插入数据
        $cData->consume_amount_tal     = $consumeAmount;
        $cData->view_tal               = $view;
        $cData->event_pay_tal          = $dataVal['event_pay'] ?? 0;
        $cData->t0_direct_paied_cnt_tal = $dataVal['t0_direct_paied_cnt'] ?? 0;
        $cData->play_tal               = $play;
        $cData->action_tal             = $action;
        $cData->conversion_tal         = $conversion;
        $cData->comment_tal            = $comment;
        $cData->admire_tal             = $admire;
        $cData->played3s_cnt_tal       = $played3sCnt;
        $cData->submissions_number_tal = $submissionsNumber;

        # 获取昨日数据信息
        $yData = $lastDataMap->get($supplementOrderId) ?? null;
        if (!empty($yData)) {
            # 如过昨日为获取到达人信息将更新昨日达人数据
            if (($yData->user_id == -1) && ($starInfo['user_id'] != -1)) {
                $yData->update([
                    'kwai_id'   => $starInfo['kwai_id'],
                    'user_id'   => $starInfo['user_id'],
                    'star_name' => $starInfo['star_name']
                ]);
            }

            # 新增数据
            $cData->consume_amount_incr    =  $consumeAmount - $yData->consume_amount_tal;
            $cData->view_incr              = $view - $yData->view_tal;
            $cData->event_pay_incr         = $cData->event_pay_tal - $yData->event_pay_tal;
            $cData->t0_direct_paied_cnt_incr = $cData->t0_direct_paied_cnt_tal - $yData->t0_direct_paied_cnt_tal;
            $cData->play_incr              = $play - $yData->play_tal;
            $cData->action_incr            = $action - $yData->action_tal;
            $cData->conversion_incr        = $conversion - $yData->conversion_tal;
            $cData->comment_incr           = $comment - $yData->comment_tal;
            $cData->admire_incr            = $admire - $yData->admire_tal;
            $cData->played3s_cnt_incr      = $played3sCnt - $yData->played3s_cnt_tal;
            $cData->submissions_number_incr = $submissionsNumber - $yData->submissions_number_tal;

            # 今日首条数据与昨日比较
            if ($cData->consume_amount_incr < 0) {
                $cData->consume_amount_tal  = $yData->consume_amount_tal;
                $cData->consume_amount_incr = 0;
            }
            if ($cData->view_incr < 0) {
                $cData->view_tal  = $yData->view_tal;
                $cData->view_incr = 0;
            }
            if ($cData->event_pay_incr < 0) {
                $cData->event_pay_tal  = $yData->event_pay_incr;
                $cData->event_pay_incr = 0;
            }
            if ($cData->t0_direct_paied_cnt_incr < 0) {
                $cData->t0_direct_paied_cnt_tal  = $yData->t0_direct_paied_cnt_tal;
                $cData->t0_direct_paied_cnt_incr = 0;
            }
            if ($cData->play_incr < 0) {
                $cData->play_tal  = $yData->play_tal;
                $cData->play_incr = 0;
            }
            if ($cData->action_incr < 0) {
                $cData->action_tal  = $yData->action_tal;
                $cData->action_incr = 0;
            }
            if ($cData->conversion_incr < 0) {
                $cData->conversion_tal  = $yData->conversion_tal;
                $cData->conversion_incr = 0;
            }
            if ($cData->comment_incr < 0) {
                $cData->comment_tal  = $yData->comment_tal;
                $cData->comment_incr = 0;
            }
            if ($cData->admire_incr < 0) {
                $cData->admire_tal  = $yData->admire_tal;
                $cData->admire_incr = 0;
            }
            if ($cData->played3s_cnt_incr < 0) {
                $cData->played3s_cnt_tal  = $yData->played3s_cnt_tal;
                $cData->played3s_cnt_incr = 0;
            }
            if ($cData->submissions_number_incr < 0) {
                $cData->submissions_number_tal  = $yData->submissions_number_tal;
                $cData->submissions_number_incr = 0;
            }
        } else {
            # 新增数据
            $cData->consume_amount_incr     = $consumeAmount;
            $cData->view_incr               = $view;
            $cData->event_pay_incr          = $dataVal['event_pay'] ?? 0;
            $cData->t0_direct_paied_cnt_incr= $dataVal['t0_direct_paied_cnt'] ?? 0;
            $cData->play_incr               = $play;
            $cData->action_incr             = $action;
            $cData->conversion_incr         = $conversion;
            $cData->comment_incr            = $comment;
            $cData->admire_incr             = $admire;
            $cData->played3s_cnt_incr       = $played3sCnt;
            $cData->submissions_number_incr = $submissionsNumber;
        }

        $cData->save();
    }

    public function orderHourDataInDb($advertiserId, $dataVal, $curHour, $starMap, $lastDataMap) {
//        Log::info('数据记录', [
//            'advertiser_id' => $advertiserId, 'data_val' => $dataVal, 'cur_hour' => $curHour, 'last_data' => $lastDataMap
//        ], 'StarVideoOrderHourDataSync');
        # 检测订单数据
        $orderId = $dataVal['order_id'] ?? null;
        if (empty($orderId)) {
            # 输出错误日志
            Log::error('adv:'.$advertiserId.' orderId 为空', $dataVal, 'Flow:StarVideoOrderHourDataSync');
            return false;
        }

        # 检测生效助推订单数
        $closingTime = $dataVal['closing_time'];
        if (
            !empty($closingTime)
            &&
            ($closingTime < date('Y-m-d H:i:s', strtotime("-12 hour")))
        ) {
            return true;
        }

        # 获取达人信息
        $taskId = $dataVal['task_id'];
        $starInfo = $starMap[$taskId][$orderId] ?? [
                'kwai_id'   => 'None',
                'user_id'   => -1,
                'star_name' => 'None'
            ];

        # 检测是否存在当前小时的数据
        $supplementOrderId = $dataVal['supplement_order_id'];
        $cData = JxStarVideoFlowOrderHourData::query()
            ->where('date', $curHour)
            ->where('advertiser_id', $advertiserId)
            ->where('supplement_order_id', $supplementOrderId)
            ->first();

        # 数据异常检测
        $consumeAmount = $dataVal['consume_amount'];
        $unitPrice     = $dataVal['unit_price'] ?? 0;
        $play          = $dataVal['play'] ?? 0;
        $action        = $dataVal['action'] ?? 0;
        $conversion    = $dataVal['conversion'] ?? 0;
        $submissionsNumber = $dataVal['submissions_number'] ?? 0;
        if (!empty($cData)) {
            if ( ($consumeAmount < $cData->consume_amount_tal)
                ||
                ($play < $cData->play_tal)
                ||
                ($action < $cData->action_tal)
                ||
                ($conversion < $cData->conversion_tal)
            ) {
                # 输出错误日志
                Log::info('adv:'.$advertiserId.' 返回数据存在异常', $dataVal, 'Flow:StarVideoOrderHourDataSync');
                return false;
            }
        } else {
            $cData = new JxStarVideoFlowOrderHourData();
            $cData->advertiser_id       = $advertiserId;
            $cData->task_id             = $taskId;
            $cData->supplement_order_id = $supplementOrderId;
            $cData->date                = $curHour;
        }

        # 达人数据更新
        if (($cData->user_id == -1) || is_null($cData->user_id)) {
            $cData->kwai_id   = $starInfo['kwai_id'];
            $cData->user_id   = $starInfo['user_id'];
            $cData->star_name = $starInfo['star_name'];
        }

        # 插入数据
        $cData->consume_amount_tal     = $consumeAmount;
        $cData->unit_price_tal         = $unitPrice;
        $cData->play_tal               = $play;
        $cData->action_tal             = $action;
        $cData->conversion_tal         = $conversion;
        $cData->submissions_number_tal = $submissionsNumber;

        # 获取昨日数据信息
        $yData = $lastDataMap->get($supplementOrderId) ?? null;
        if (!empty($yData)) {
            # 如过昨日为获取到达人信息将更新昨日达人数据
            if (($yData->user_id == -1) && ($starInfo['user_id'] != -1)) {
                $yData->update([
                    'kwai_id'   => $starInfo['kwai_id'],
                    'user_id'   => $starInfo['user_id'],
                    'star_name' => $starInfo['star_name']
                ]);
            }

            # 新增数据
            $cData->consume_amount_incr    = $consumeAmount - $yData->consume_amount_tal;
            $cData->unit_price_incr        = $unitPrice - $yData->unit_price_tal;
            $cData->play_incr              = $play - $yData->play_tal;
            $cData->action_incr            = $action - $yData->action_tal;
            $cData->conversion_incr        = $conversion - $yData->conversion_tal;

            # 今日首条数据与昨日比较
            if ($cData->consume_amount_incr < 0) {
                $cData->consume_amount_tal  = $yData->consume_amount_tal;
                $cData->consume_amount_incr = 0;
            }
            if ($cData->play_incr < 0) {
                $cData->play_tal  = $yData->play_tal;
                $cData->play_incr = 0;
            }
            if ($cData->action_incr < 0) {
                $cData->action_tal  = $yData->action_tal;
                $cData->action_incr = 0;
            }
            if ($cData->conversion_incr < 0) {
                $cData->conversion_tal  = $yData->conversion_tal;
                $cData->conversion_incr = 0;
            }
            if ($cData->submissions_number_incr < 0) {
                $cData->submissions_number_tal  = $yData->submissions_number_tal;
                $cData->submissions_number_incr = 0;
            }
        } else {
            # 新增数据
            $cData->consume_amount_incr     = $consumeAmount;
            $cData->unit_price_incr         = $unitPrice;
            $cData->play_incr               = $play;
            $cData->action_incr             = $action;
            $cData->conversion_incr         = $conversion;
            $cData->submissions_number_incr = $submissionsNumber;
        }

        $cData->save();
    }
}
