<?php

namespace App\Http\Controllers\Api\Sys;

use App\Http\Controllers\Controller;
use App\Services\Sys\StarLibraryService;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class StarLibraryController extends Controller
{
    /**
     * 达人信息导入
     * @param Request $request
     * @return array
     */
    public function infoUpload(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'ptype' => 'required|int',
            'file'  => 'required|file'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $ptype = $request->input('ptype');
        $file  = $request->file('file');

        list($res, $code) = StarLibraryService::infoUpload($ptype, $file);
        return self::returnValue($res, $code);
    }

    /**
     * 达人机构导入
     * @param Request $request
     * @return array
     */
    public function instUpload(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'file'  => 'required|file'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $file  = $request->file('file');

        list($res, $code) = StarLibraryService::instUpload($file);
        return self::returnValue($res, $code);
    }

    /**
     * 类别列表
     * @param Request $request
     * @return array
     */
    public function category(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'ptype' => ['required',Rule::in([0, 1, 2, 3])]
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $ptype = $request->input('ptype');

        list($res, $code) = StarLibraryService::category($ptype);
        return self::returnValue($res, $code);
    }

    /**
     * 达人信息列表
     * @param Request $request
     * @return array
     */
    public function infoList(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'ptype'      => 'required|int',
            'cat_id'     => 'nullable|string',
            'ins_id'     => 'nullable|string',
            'min_fans'   => 'nullable|numeric',
            'max_fans'   => 'nullable|numeric',
            'min_qprice' => 'nullable|int',
            'max_qprice' => 'nullable|int',
            'keyword'    => 'nullable|string'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $ptype     = $request->input('ptype');
        $catId     = $request->input('cat_id');
        $insId     = $request->input('ins_id');
        $minFans   = $request->input('min_fans');
        $maxFans   = $request->input('max_fans');
        $minQprice = $request->input('min_qprice');
        $maxQprice = $request->input('max_qprice');
        $keyword   = $request->input('keyword');
        $sortField = $request->input('sort_field', 'created_at');
        $sortOrder = $request->input('sort_order', 'desc');
        $page      = $request->input('page', 1);
        $pageSize  = $request->input('page_size', 20);

        list($list, $total, $extra) = StarLibraryService::infoList(
            $ptype, $catId, $insId, $minFans, $maxFans, $minQprice,
            $maxQprice, $keyword, $sortField, $sortOrder, $page, $pageSize
        );
        return self::returnPageValue($list, $total, $page, $pageSize, $extra);
    }

    /**
     * 达人机构列表
     * @param Request $request
     * @return array
     */
    public function instList(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'cat_id'     => 'nullable|string',
            'keyword'    => 'nullable|string'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $catId    = $request->input('cat_id');
        $keyword  = $request->input('keyword');
        $page     = $request->input('page', 1);
        $pageSize = $request->input('page_size',20);

        list($list, $total, $extra) = StarLibraryService::instList($catId, $keyword, $page, $pageSize);
        return self::returnPageValue($list, $total, $page, $pageSize, $extra);
    }

    /**
     * 上传记录列表
     * @param Request $request
     * @return array
     */
    public function upList(Request $request)
    {
        $page     = $request->input('page', 1);
        $pageSize = $request->input('page_size',20);

        list($list, $total) = StarLibraryService::upList($page, $pageSize);
        return self::returnPageValue($list, $total, $page, $pageSize);
    }

    /**
     * 用户组列表
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function groupList(Request $request)
    {
        $res = StarLibraryService::groupList();
        return self::returnValue($res);
    }

    /**
     * 用户组添加
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function groupAdd(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'name' => 'required|string'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $name = $request->input('name');

        list($res, $code) = StarLibraryService::groupAdd($name);
        return self::returnValue($res, $code);
    }

    /**
     * 用户组编辑
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function groupEdit(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'group_id' => 'required|int',
            'name'     => 'required|string'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $groupId = $request->input('group_id');
        $name = $request->input('name');

        list($res, $code) = StarLibraryService::groupEdit($groupId, $name);
        return self::returnValue($res, $code);
    }

    /**
     * 用户组删除
     * @param \Illuminate\Http\Request $request
     * @return array
     */
    public function groupDel(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'group_id' => 'required|int'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 100);
        }

        $groupId = $request->input('group_id');

        list($res, $code) = StarLibraryService::groupDel($groupId);
        return self::returnValue($res, $code);
    }
}
